﻿//-----------------------------------------------------------------------------
//
//  mainForm.cs
//
//  PC Case USB LCD
//  (USB Generic HID Communications 2_0_0_0)
//
//  A USB driven LCD screen for use in PC cases
//  Copyright (C) 2010 Simon Inns
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  Web:    http://www.waitingforfriday.com
//  Email:  simon.inns@gmail.com
//
//-----------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace PC_Case_USB_LCD
    {
    public partial class mainForm : Form
        {
        public mainForm()
            {
            InitializeComponent();
            // Create the PC Case USB LCD object (passing VID and PID)
            thepcCaseUsbLCD = new pcCaseUsbLCD(0x04D8, 0x0081);

            // Register for device change notifications
            thepcCaseUsbLCD.registerForDeviceNotifications(this.Handle);

            // Add a listener for usb events
            thepcCaseUsbLCD.usbEvent += new pcCaseUsbLCD.usbEventsHandler(usbEvent_receiver);

            // Perform an initial search for the target device
            thepcCaseUsbLCD.findTargetDevice();
            }

        // Create an instance of the PC Case USB LCD
        private pcCaseUsbLCD thepcCaseUsbLCD;

        // Callback method for WndProc
        protected override void WndProc(ref Message m)
            {
            // Pass the message to our message handler
            thepcCaseUsbLCD.handleDeviceNotificationMessages(m);

            // Pass the message on to the base WndProc
            base.WndProc(ref m);
            }

        // Listener for USB events
        private void usbEvent_receiver(object o, EventArgs e)
            {
            // Check the status of the USB device and update the form accordingly
            if (thepcCaseUsbLCD.isDeviceAttached)
                {
                // Device is currently attached

                // Update the status label
                this.usbStatusLabel.Text = "LCD is connected";

                // Clear the LCD
                thepcCaseUsbLCD.clearDisplay();
                }
            else
                {
                // Device is currently unattached

                // Update the status label
                this.usbStatusLabel.Text = "LCD is not connected";
                }
            }

        private void debugCollectionTimer_Tick(object sender, EventArgs e)
            {
            String debugString;

            // Only collect debug if the device is attached
            if (thepcCaseUsbLCD.isDeviceAttached)
                {
                // Collect the debug information from the device
                debugString = thepcCaseUsbLCD.collectDebug();

                // Display the debug information
                if (debugString != String.Empty)
                    {
                    this.debugTextBox.AppendText(debugString);
                    }
                }
            else
                {
                // Clear the debug window
                this.debugTextBox.Clear();
                }
            }

        // The display update timer has ticked
        private void displayUpdateTimer_Tick(object sender, EventArgs e)
            {
            Int32 cpuUsage;
            Int32 memoryUsage;

            // Read the CPU usage performance counter value
            try
                {
                cpuUsage = Convert.ToInt32(cpuPerformanceCounter.NextValue());

                // Range check
                if (cpuUsage > 100) cpuUsage = 100;
                }
            catch
                {
                // If this fails we just default to disabled
                cpuUsage = -1;
                }

            // Read the memory usage performance counter value
            try
                {
                memoryUsage = Convert.ToInt32(memoryPerformanceCounter.NextValue());

                // Range check
                if (memoryUsage > 100) cpuUsage = memoryUsage;
                }
            catch
                {
                // If this fails we just default to disabled
                memoryUsage = -1;
                }

            // Write the performance counter values to the main form
            this.cpuUsageValueLabel.Text = String.Concat(Convert.ToString(cpuUsage), " %");
            this.memoryUsageValueLabel.Text = String.Concat(Convert.ToString(memoryUsage), " %");

            // Write the performance counter values to the LCD
            thepcCaseUsbLCD.moveCursor(0, 0);
            thepcCaseUsbLCD.outputString(String.Concat("CPU:", Convert.ToString(cpuUsage), "%  "));

            thepcCaseUsbLCD.moveCursor(8, 0);
            thepcCaseUsbLCD.outputString(String.Concat("Mem:", Convert.ToString(memoryUsage), "%  "));

            // Display the current time and date
            DateTime time = new DateTime();

            // Get the current time and date
            time = DateTime.Now;

            String clock;

            // Pad single digits with a space to make the output look nice
            if (time.Hour < 10) clock = String.Concat("0", Convert.ToString(time.Hour), ":");
            else clock = String.Concat(Convert.ToString(time.Hour),":");

            if (time.Minute < 10) clock = String.Concat(clock, "0", Convert.ToString(time.Minute), ":");
            else clock = String.Concat(clock, Convert.ToString(time.Minute), ":");

            if (time.Second < 10) clock = String.Concat(clock, "0", Convert.ToString(time.Second));
            else clock = String.Concat(clock, Convert.ToString(time.Second));

            thepcCaseUsbLCD.moveCursor(4, 1);
            thepcCaseUsbLCD.outputString(clock);
            }
        }
    }
